;; absolute-s/theme.jl
;; $Id$

;; Copyright (C) 1999 Mike Bond <mbond@erols.com>

;; This file is part of sawmill.

;; sawmill is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; sawmill is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with sawmill; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; The basic bar images and design of this theme are from Hallvar Helleseth's
;; Absolute E theme for Enlightenment. The original theme.jl is from
;; John Harper's absolute-e theme for sawmill.

;; Note: The button and foreground images were used along with gimp and it's
;;	 layers to create the close, iconify and maximize buttons. I've kept
;;	 the originals around as someone may want to play with the buttons
;;	 some more. In particular, I'm still not real happy with the close
;;	 button, I tried an x and it was horrible, other variants weren't so
;;	 bad but this is the one I've liked the most so far.
;;
;; sawmill --batch --no-rc compiler -f compile-lib-batch .

;; from the ABOUT file:

;;    Thanks for using Absolute E, the theme that thinks diffrently.
;;    This theme is created by Hallvar Helleseth using nothing but The
;;    Gimp.  Everything is originaly made by me - Propably the only
;;    Etheme that can say that ;) Though alot of people have helped me
;;    (too many to list here/can't remeber all of them either...) But I
;;    give them a BIG thank you!

;;    There is a GTK and WMaker version of this theme available at
;;    themes.org

;;    This theme is going to be following the cvs version of E and not
;;    the stable version (DR0.15.5)

;;    Reach me at hallvar@ii.uib.no (www.ii.uib.no/~hallvar)

;;    Date: July 19 1999.

;; preferences

(defgroup absolute-s-frame "Absolute Sawmill")

(defcustom absolute-s:foreground "white"
  "Title foreground color"
  :group absolute-s-frame
  :type color
  :after-set absolute-s:rebuild)

(defcustom absolute-s:full-title nil
  "Use full width of window for title and buttons"
  :group absolute-s-frame
  :type boolean
  :after-set absolute-s:rebuild)

(defcustom absolute-s:border-dimension 4
  "Width of window border"
  :group absolute-s-frame
  :type number
  :range (1 . nil)
  :after-set absolute-s:rebuild)

(defun absolute-s:rebuild ()
  (mapc 'rebuild-frame (managed-windows)))

;; 100x16
(defvar absolute-s:bar-images
  (mapcar #'(lambda (i)
	      (set-image-border i 4 4 4 4))
	  (list (make-image "bar_normal.png")
		(make-image "bar_normal_active.png")
		(make-image "bar_hilited_active.png")
		(make-image "bar_clicked_active.png"))))

(defvar absolute-s:close-images
  (mapcar #'(lambda (i)
	      (set-image-border i 4 4 4 4))
	  (list (make-image "close_normal.png")
		(make-image "close_normal_active.png")
		(make-image "close_hilited_active.png")
		(make-image "close_clicked_active.png"))))

(defvar absolute-s:iconify-images
  (mapcar #'(lambda (i)
	      (set-image-border i 4 4 4 4))
	  (list (make-image "iconify_normal.png")
		(make-image "iconify_normal_active.png")
		(make-image "iconify_hilited_active.png")
		(make-image "iconify_clicked_active.png"))))

(defvar absolute-s:maximize-images
  (mapcar #'(lambda (i)
	      (set-image-border i 4 4 4 4))
	  (list (make-image "maximize_normal.png")
		(make-image "maximize_normal_active.png")
		(make-image "maximize_hilited_active.png")
		(make-image "maximize_clicked_active.png"))))

(defun absolute-s:border-dimension () absolute-s:border-dimension)
(defun absolute-s:-border-dimension () (- absolute-s:border-dimension))
(defun absolute-s:foreground () absolute-s:foreground)

(defun absolute-s:title-width (w)
  (let
      ((w-width (car (window-dimensions w))))
    (cond ((eq absolute-s:full-title 'nil)
           (max 0 (min (- w-width 64) (max (- (/ w-width 2) 64) (+ 10 (text-width (window-name w)))))))
          (t
	   (max 0 (- w-width 64))))))

(defun absolute-s:update-text-width (w atom type)
  (if (eq atom 'WM_NAME)
      (rebuild-frame w)))

(defun absolute-s:init ()
  (add-hook 'property-notify-hook 'absolute-s:update-text-width))

(defvar absolute-s:frame
  `(;; menu button
    ((background . ,absolute-s:bar-images)
     (left-edge . 0)
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . menu-button))
    ;; title bar
    ((background . ,absolute-s:bar-images)
     (foreground . absolute-s:foreground)
     (text . window-name)
     (x-justify . center)
     (y-justify . center)
     (left-edge . 16)
     (width . absolute-s:title-width)
     (top-edge . -16)
     (class . title))
    ;; iconify button
    ((background . ,absolute-s:iconify-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 16)))
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . iconify-button))
    ;; maximize button
    ((background . ,absolute-s:maximize-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 32)))
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . maximize-button))
    ;; close button
    ((background . ,absolute-s:close-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 48)))
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . close-button))
    ;; rhs bit of title
    ((background . ,absolute-s:bar-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 64)))
     (right-edge . 0)
     (top-edge . absolute-s:-border-dimension)
     (height . absolute-s:border-dimension)
     (class . top-border))
    ;; left frame
    ((background . ,absolute-s:bar-images)
     (right-edge . absolute-s:-border-dimension)
     (width . absolute-s:border-dimension)
     (top-edge . 0)
     (bottom-edge . 0)
     (class . right-border))
    ;; right frame
    ((background . ,absolute-s:bar-images)
     (left-edge . absolute-s:-border-dimension)
     (width . absolute-s:border-dimension)
     (top-edge . 0)
     (bottom-edge . 0)
     (class . left-border))
    ;; bottom frame
    ((background . ,absolute-s:bar-images)
     (left-edge . 0)
     (right-edge . 0)
     (bottom-edge . absolute-s:-border-dimension)
     (height . absolute-s:border-dimension)
     (class . bottom-border))
    ;; top-left corner
    ((background . ,absolute-s:bar-images)
     (left-edge . absolute-s:-border-dimension)
     (width . absolute-s:border-dimension)
     (top-edge . absolute-s:-border-dimension)
     (height . absolute-s:border-dimension)
     (class . top-left-corner))
    ;; top-right corner
    ((background . ,absolute-s:bar-images)
     (right-edge . absolute-s:-border-dimension)
     (width . absolute-s:border-dimension)
     (top-edge . absolute-s:-border-dimension)
     (height . absolute-s:border-dimension)
     (class . top-right-corner))
    ;; bottom-left corner
    ((background . ,absolute-s:bar-images)
     (left-edge . absolute-s:-border-dimension)
     (width . absolute-s:border-dimension)
     (bottom-edge . absolute-s:-border-dimension)
     (height . absolute-s:border-dimension)
     (class . bottom-left-corner))
    ;; bottom-right corner
    ((background . ,absolute-s:bar-images)
     (right-edge . absolute-s:-border-dimension)
     (width . absolute-s:border-dimension)
     (bottom-edge . absolute-s:-border-dimension)
     (height . absolute-s:border-dimension)
     (class . bottom-right-corner))))

(defvar absolute-s:shaped-frame
 `(;; menu button
   ((background . ,absolute-s:bar-images)
    (left-edge . 0)
    (width . 16)
    (top-edge . -16)
    (height . 16)
    (class . menu-button))
   ;; title bar
   ((background . ,absolute-s:bar-images)
    (foreground . absolute-s:foreground)
    (text . window-name)
    (x-justify . center)
    (y-justify . center)
    (left-edge . 16)
    (width . absolute-s:title-width)
    (top-edge . -16)
    (class . title))
    ;; iconify button
    ((background . ,absolute-s:iconify-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 16)))
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . iconify-button))
    ;; maximize button
    ((background . ,absolute-s:maximize-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 32)))
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . maximize-button))
    ;; close button
    ((background . ,absolute-s:close-images)
     (left-edge . (lambda (w)
		    (+ (absolute-s:title-width w) 48)))
     (width . 16)
     (top-edge . -16)
     (height . 16)
     (class . close-button))
   ;; rhs bit of title
   ((background . ,absolute-s:bar-images)
    (left-edge . (lambda (w)
		   (+ (absolute-s:title-width w) 64)))
    (right-edge . 0)
    (top-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . title))))

(defvar absolute-s:transient-frame
 `(((background . ,absolute-s:bar-images)
    (left-edge . 0)
    (right-edge . 0)
    (top-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . title))
   ((background . ,absolute-s:bar-images)
    (left-edge . 0)
    (right-edge . 0)
    (bottom-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . bottom-border))
   ((background . ,absolute-s:bar-images)
    (left-edge . absolute-s:-border-dimension)
    (width . absolute-s:border-dimension)
    (top-edge . 0)
    (bottom-edge . 0)
    (class . left-border))
   ((background . ,absolute-s:bar-images)
    (right-edge . absolute-s:-border-dimension)
    (width . absolute-s:border-dimension)
    (top-edge . 0)
    (bottom-edge . 0)
    (class . right-border))
   ;; top-right corner
   ((background . ,absolute-s:bar-images)
    (right-edge . absolute-s:-border-dimension)
    (width . absolute-s:border-dimension)
    (top-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . top-right-corner))
   ;; top-left corner
   ((background . ,absolute-s:bar-images)
    (left-edge . absolute-s:-border-dimension)
    (width . absolute-s:border-dimension)
    (top-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . top-left-corner))
   ;; bottom-left corner
   ((background . ,absolute-s:bar-images)
    (left-edge . absolute-s:-border-dimension)
    (width . absolute-s:border-dimension)
    (bottom-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . bottom-left-corner))
   ;; bottom-right corner
   ((background . ,absolute-s:bar-images)
    (right-edge . absolute-s:-border-dimension)
    (width . absolute-s:border-dimension)
    (bottom-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . bottom-right-corner))))

(defvar absolute-s:shaped-transient-frame
 `(((background . ,absolute-s:bar-images)
    (left-edge . 0)
    (right-edge . 0)
    (top-edge . absolute-s:-border-dimension)
    (height . absolute-s:border-dimension)
    (class . title))))

(defun absolute-s:frame-style (w type)
  (cond ((eq type 'shaped)
	 'absolute-s:shaped-frame)
	((eq type 'transient)
	 'absolute-s:transient-frame)
	((eq type 'shaped-transient)
	 'absolute-s:shaped-transient-frame)
	((eq type 'unframed)
	 'nil-frame)
	(t
	 'absolute-s:frame)))

(add-frame-style 'absolute-s 'absolute-s:frame-style)


;; initialization

(unless batch-mode
  (absolute-s:init))
